@@ -8,12 +8,9 @@ Routines for testing WSGI applications.
 Most interesting is TestApp
 """
 
-import sys
 import random
-import urllib
 import warnings
 import mimetypes
-import time
 import cgi
 import os
 import re
@@ -33,6 +30,7 @@ from webtest.compat import text_type
 from webtest.compat import to_string
 from webtest.compat import to_bytes
 from webtest.compat import join_bytes
+from webtest.compat import OrderedDict
 from webtest.compat import dumps
 from webtest.compat import loads
 from webtest.compat import PY3
@@ -41,7 +39,7 @@ from webob import Request, Response
 if PY3:
     from webtest import lint3 as lint
 else:
-    from webtest import lint
+    from webtest import lint  # NOQA
 
 __all__ = ['TestApp', 'TestRequest']
 
@@ -51,28 +49,26 @@ class NoDefault(object):
 
 
 class AppError(Exception):
-    pass
 
-
-class CaptureStdout(object):
-
-    def __init__(self, actual):
-        self.captured = StringIO()
-        self.actual = actual
-
-    def write(self, s):
-        self.captured.write(s)
-        self.actual.write(s)
-
-    def flush(self):
-        self.actual.flush()
-
-    def writelines(self, lines):
-        for item in lines:
-            self.write(item)
-
-    def getvalue(self):
-        return self.captured.getvalue()
+    def __init__(self, message, *args):
+        message = to_string(message)
+        str_args = ()
+        for arg in args:
+            if isinstance(arg, Response):
+                body = arg.body
+                if isinstance(body, binary_type):
+                    if arg.charset:
+                        arg = body.decode(arg.charset)
+                    else:
+                        arg = repr(body)
+            elif isinstance(arg, binary_type):
+                try:
+                    arg = to_string(arg)
+                except UnicodeDecodeError:
+                    arg = repr(arg)
+            str_args += (arg,)
+        message = message % str_args
+        Exception.__init__(self, message)
 
 
 class TestResponse(Response):
@@ -806,8 +802,8 @@ class TestApp(object):
                                  expect_errors=expect_errors,
                                  content_type=content_type)
 
-    def post_json(self, url, params='', headers=None, extra_environ=None,
-                  status=None, expect_errors=False):
+    def post_json(self, url, params=NoDefault, headers=None,
+                  extra_environ=None, status=None, expect_errors=False):
         """
         Do a POST request.  Very like the ``.get()`` method.
         ``params`` are dumps to json and put in the body of the request.
@@ -816,7 +812,7 @@ class TestApp(object):
         Returns a ``webob.Response`` object.
         """
         content_type = 'application/json'
-        if params:
+        if params is not NoDefault:
             params = dumps(params)
         return self._gen_request('POST', url, params=params, headers=headers,
                                  extra_environ=extra_environ, status=status,
@@ -842,7 +838,7 @@ class TestApp(object):
                                  expect_errors=expect_errors,
                                  content_type=content_type)
 
-    def put_json(self, url, params='', headers=None, extra_environ=None,
+    def put_json(self, url, params=NoDefault, headers=None, extra_environ=None,
             status=None, expect_errors=False):
         """
         Do a PUT request.  Very like the ``.post()`` method.
@@ -852,7 +848,7 @@ class TestApp(object):
         Returns a ``webob.Response`` object.
         """
         content_type = 'application/json'
-        if params:
+        if params is not NoDefault:
             params = dumps(params)
         return self._gen_request('PUT', url, params=params, headers=headers,
                                  extra_environ=extra_environ, status=status,
@@ -877,8 +873,8 @@ class TestApp(object):
                                  expect_errors=expect_errors,
                                  content_type=content_type)
 
-    def delete_json(self, url, params='', headers=None, extra_environ=None,
-               status=None, expect_errors=False):
+    def delete_json(self, url, params=NoDefault, headers=None,
+                    extra_environ=None, status=None, expect_errors=False):
         """
         Do a DELETE request.  Very like the ``.get()`` method.
         Content-Type is set to ``application/json``.
@@ -890,7 +886,7 @@ class TestApp(object):
                            'DELETE request. Most web servers will ignore it'),
                            lint.WSGIWarning)
         content_type = 'application/json'
-        if params:
+        if params is not NoDefault:
             params = dumps(params)
         return self._gen_request('DELETE', url, params=params, headers=headers,
                                  extra_environ=extra_environ, status=status,
@@ -1043,17 +1039,9 @@ class TestApp(object):
         req.environ['paste.testing'] = True
         req.environ['paste.testing_variables'] = {}
         app = lint.middleware(self.app)
-        old_stdout = sys.stdout
-        out = CaptureStdout(old_stdout)
-        try:
-            sys.stdout = out
-            start_time = time.time()
-            ## FIXME: should it be an option to not catch exc_info?
-            res = req.get_response(app, catch_exc_info=True)
-            res._use_unicode = self.use_unicode
-            end_time = time.time()
-        finally:
-            sys.stdout = old_stdout
+        ## FIXME: should it be an option to not catch exc_info?
+        res = req.get_response(app, catch_exc_info=True)
+        res._use_unicode = self.use_unicode
         res.request = req
         res.app = app
         res.test_app = self
@@ -1063,7 +1051,6 @@ class TestApp(object):
         except TypeError:
             pass
         res.errors = errors.getvalue()
-        total_time = end_time - start_time
         for name, value in req.environ['paste.testing_variables'].items():
             if hasattr(res, name):
                 raise ValueError(
@@ -1098,26 +1085,26 @@ class TestApp(object):
         if isinstance(status, (list, tuple)):
             if res.status_int not in status:
                 raise AppError(
-                    "Bad response: %s (not one of %s for %s)\n%s"
-                    % (res_status, ', '.join(map(str, status)),
-                       res.request.url, res.body))
+                    "Bad response: %s (not one of %s for %s)\n%s",
+                    res_status, ', '.join(map(str, status)),
+                    res.request.url, res)
             return
         if status is None:
             if res.status_int >= 200 and res.status_int < 400:
                 return
             raise AppError(
-                "Bad response: %s (not 200 OK or 3xx redirect for %s)\n%s"
-                % (res_status, res.request.url,
-                   res.body))
+                "Bad response: %s (not 200 OK or 3xx redirect for %s)\n%s",
+                res_status, res.request.url,
+                res)
         if status != res.status_int:
             raise AppError(
-                "Bad response: %s (not %s)" % (res_status, status))
+                "Bad response: %s (not %s)", res_status, status)
 
     def _check_errors(self, res):
         errors = res.errors
         if errors:
             raise AppError(
-                "Application had errors logged:\n%s" % errors)
+                "Application had errors logged:\n%s", errors)
 
 
 ########################################
@@ -1458,7 +1445,7 @@ class Form(object):
     def _parse_fields(self):
         in_select = None
         in_textarea = None
-        fields = {}
+        fields = OrderedDict()
         for match in self._tag_re.finditer(self.text):
             end = match.group(1) == '/'
             tag = match.group(2).lower()
@@ -1768,7 +1755,10 @@ def html_unquote(v):
         v = v.replace(ent, repl)
     return v
 
+
 def encode_params(params, content_type):
+    if params is NoDefault:
+        return ''
     if isinstance(params, dict) or hasattr(params, 'items'):
         params = list(params.items())
     if isinstance(params, (list, tuple)):
